@@ -37,6 +37,7 @@ HANDLE PASCAL RAROpenArchiveEx(struct RAROpenArchiveDataEx *r)
   {
     r->OpenResult=0;
     DataSet *Data=new DataSet;
+    Data->Cmd.DllError=0;
     Data->OpenMode=r->OpenMode;
     Data->Cmd.FileArgs->AddString("*");
 
@@ -49,6 +50,7 @@ HANDLE PASCAL RAROpenArchiveEx(struct RAROpenArchiveDataEx *r)
 
     Data->Cmd.AddArcName(r->ArcName,r->ArcNameW);
     Data->Cmd.Overwrite=OVERWRITE_ALL;
+    Data->Cmd.VersionControl=1;
     if (!Data->Arc.Open(r->ArcName,r->ArcNameW))
     {
       delete Data;
@@ -58,12 +60,12 @@ HANDLE PASCAL RAROpenArchiveEx(struct RAROpenArchiveDataEx *r)
     if (!Data->Arc.IsArchive(false))
     {
       delete Data;
-      r->OpenResult=ERAR_BAD_ARCHIVE;
+      r->OpenResult=Data->Cmd.DllError!=0 ? Data->Cmd.DllError:ERAR_BAD_ARCHIVE;
       return(NULL);
     }
     r->Flags=Data->Arc.NewMhd.Flags;
     Array<byte> CmtData;
-    if (r->CmtBufSize!=0 && Data->Arc.GetComment(CmtData))
+    if (r->CmtBufSize!=0 && Data->Arc.GetComment(&CmtData,NULL))
     {
       r->Flags|=2;
       int Size=CmtData.Size()+1;
@@ -117,8 +119,11 @@ int PASCAL RARReadHeader(HANDLE hArcData,struct RARHeaderData *D)
     }
     if (Data->OpenMode==RAR_OM_LIST && (Data->Arc.NewLhd.Flags & LHD_SPLIT_BEFORE))
     {
-      if (RARProcessFile(hArcData,RAR_SKIP,NULL,NULL)==0)
+      int Code=RARProcessFile(hArcData,RAR_SKIP,NULL,NULL);
+      if (Code==0)
         return(RARReadHeader(hArcData,D));
+      else
+        return(Code);
     }
     strncpy(D->ArcName,Data->Arc.FileName,sizeof(D->ArcName));
     strncpy(D->FileName,Data->Arc.NewLhd.FileName,sizeof(D->FileName));
@@ -162,8 +167,11 @@ int PASCAL RARReadHeaderEx(HANDLE hArcData,struct RARHeaderDataEx *D)
     }
     if (Data->OpenMode==RAR_OM_LIST && (Data->Arc.NewLhd.Flags & LHD_SPLIT_BEFORE))
     {
-      if (RARProcessFile(hArcData,RAR_SKIP,NULL,NULL)==0)
+      int Code=RARProcessFile(hArcData,RAR_SKIP,NULL,NULL);
+      if (Code==0)
         return(RARReadHeaderEx(hArcData,D));
+      else
+        return(Code);
     }
     strncpy(D->ArcName,Data->Arc.FileName,sizeof(D->ArcName));
     if (*Data->Arc.FileNameW)
@@ -174,7 +182,15 @@ int PASCAL RARReadHeaderEx(HANDLE hArcData,struct RARHeaderDataEx *D)
     if (*Data->Arc.NewLhd.FileNameW)
       strncpyw(D->FileNameW,Data->Arc.NewLhd.FileNameW,sizeof(D->FileNameW));
     else
+    {
+#ifdef _WIN_32
+      char AnsiName[NM];
+      OemToChar(Data->Arc.NewLhd.FileName,AnsiName);
+      CharToWide(AnsiName,D->FileNameW);
+#else
       CharToWide(Data->Arc.NewLhd.FileName,D->FileNameW);
+#endif
+    }
     D->Flags=Data->Arc.NewLhd.Flags;
     D->PackSize=Data->Arc.NewLhd.PackSize;
     D->PackSizeHigh=Data->Arc.NewLhd.HighPackSize;
@@ -197,7 +213,7 @@ int PASCAL RARReadHeaderEx(HANDLE hArcData,struct RARHeaderDataEx *D)
 }
 
 
-int PASCAL RARProcessFile(HANDLE hArcData,int Operation,char *DestPath,char *DestName)
+int PASCAL ProcessFile(HANDLE hArcData,int Operation,char *DestPath,char *DestName,wchar *DestPathW,wchar *DestNameW)
 {
   DataSet *Data=(DataSet *)hArcData;
   try
@@ -220,9 +236,39 @@ int PASCAL RARProcessFile(HANDLE hArcData,int Operation,char *DestPath,char *Des
     else
     {
       Data->Cmd.DllOpMode=Operation;
-      OemToChar(NullToEmpty(DestPath),Data->Cmd.ExtrPath);
-      AddEndSlash(Data->Cmd.ExtrPath);
-      OemToChar(NullToEmpty(DestName),Data->Cmd.DllDestName);
+
+      if (DestPath!=NULL || DestName!=NULL)
+      {
+#ifdef _WIN_32
+        OemToChar(NullToEmpty(DestPath),Data->Cmd.ExtrPath);
+#else
+        strcpy(Data->Cmd.ExtrPath,NullToEmpty(DestPath));
+#endif
+        AddEndSlash(Data->Cmd.ExtrPath);
+#ifdef _WIN_32
+        OemToChar(NullToEmpty(DestName),Data->Cmd.DllDestName);
+#else
+        strcpy(Data->Cmd.DllDestName,NullToEmpty(DestName));
+#endif
+      }
+      else
+      {
+        *Data->Cmd.ExtrPath=0;
+        *Data->Cmd.DllDestName=0;
+      }
+
+      if (DestPathW!=NULL || DestNameW!=NULL)
+      {
+        strncpyw(Data->Cmd.ExtrPathW,NullToEmpty(DestPathW),NM-2);
+        AddEndSlash(Data->Cmd.ExtrPathW);
+        strncpyw(Data->Cmd.DllDestNameW,NullToEmpty(DestNameW),NM-1);
+      }
+      else
+      {
+        *Data->Cmd.ExtrPathW=0;
+        *Data->Cmd.DllDestNameW=0;
+      }
+
       strcpy(Data->Cmd.Command,Operation==RAR_EXTRACT ? "X":"T");
       Data->Cmd.Test=Operation!=RAR_EXTRACT;
       bool Repeat=false;
@@ -244,6 +290,18 @@ int PASCAL RARProcessFile(HANDLE hArcData,int Operation,char *DestPath,char *Des
 }
 
 
+int PASCAL RARProcessFile(HANDLE hArcData,int Operation,char *DestPath,char *DestName)
+{
+  return(ProcessFile(hArcData,Operation,DestPath,DestName,NULL,NULL));
+}
+
+
+int PASCAL RARProcessFileW(HANDLE hArcData,int Operation,wchar *DestPath,wchar *DestName)
+{
+  return(ProcessFile(hArcData,Operation,NULL,NULL,DestPath,DestName));
+}
+
+
 void PASCAL RARSetChangeVolProc(HANDLE hArcData,CHANGEVOLPROC ChangeVolProc)
 {
   DataSet *Data=(DataSet *)hArcData;
