@@ -39,11 +39,11 @@ int main(int argc, char *argv[])
 #endif
 
 #ifdef _WIN_32
-  SetErrorMode(SEM_FAILCRITICALERRORS|SEM_NOOPENFILEERRORBOX);
+  SetErrorMode(SEM_NOALIGNMENTFAULTEXCEPT|SEM_FAILCRITICALERRORS|SEM_NOOPENFILEERRORBOX);
 
 
 #endif
-  bool ShutdownOnClose;
+
 
 #ifdef ALLOW_EXCEPTIONS
   try 
